//+------------------------------------------------------------------+
//|                                                Mogalef Bands.mq4 |
//|                      Copyright  2011, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net"

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1 Red
#property indicator_color2 Green
#property indicator_color3 Blue

extern string TimeFrame = "Current time frame";
extern int    lp = 30;
extern int    sp = 30;

double hiband[];
double median[];
double loband[];
double LinReg[];
double StdDev[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
    string short_name;
    short_name="Mogalef Bands";
    IndicatorShortName(short_name);
    //---- indicators
    IndicatorBuffers(5);  
//---- indicators
   SetIndexStyle(0, DRAW_LINE, STYLE_DOT);
   SetIndexBuffer(0, hiband);
   SetIndexStyle(1, DRAW_LINE, STYLE_DOT);
   SetIndexBuffer(1, loband);
   SetIndexStyle(2, DRAW_LINE, STYLE_DOT);
   SetIndexBuffer(2, median);
   SetIndexBuffer(3, LinReg);
   SetIndexBuffer(4, StdDev);
   //
    //
    //
    //
    //
   
    indicatorFileName = WindowExpertName();
    returnBars        = TimeFrame == "returnBars";     if (returnBars)     return(0);
    timeFrame         = stringToTimeFrame(TimeFrame);
   
    //
    //
    //
    //
    //
    
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+

int start() 
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) {  hiband[0] = limit+1; return(0); }
            if (timeFrame!=Period())
            {
               limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
               for (int i=limit; i>=0; i--)
               {
                   int y = iBarShift(NULL,timeFrame,Time[i]);               
                      hiband[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",lp,sp,0,y);
                      loband[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",lp,sp,1,y);  
                      median[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",lp,sp,2,y);           
               }
               return(0);
            }
   CalcValues(limit);

   limit = MathMin(Bars - sp, limit);
   for (int index = limit; index >=0; index--)
   {
      // band values
      if (index>(Bars-sp-lp)) StdDev[index]=0;
      hiband[index] = LinReg[index] + StdDev[index]*2;
      loband[index] = LinReg[index] - StdDev[index]*2;
      /**/
      
         if (LinReg[index] < hiband[index + 1] && LinReg[index] > loband[index+1])
         {
//            StdDev[index] = StdDev[index+1];
            hiband[index] = hiband[index+1];
            loband[index] = loband[index+1];
         }
      
      median[index] = (hiband[index] + loband[index]) / 2;
      /* */
   } 
   return(0);
  }
//+------------------------------------------------------------------+

void CalcValues(int limit)
{
   int    counted_bars=IndicatorCounted();
 
   if(counted_bars<0) counted_bars=0;
   if(counted_bars>0) counted_bars--;   
   int index = 0;
   
   LinReg[0] = LinReg[1];
   StdDev[0] = StdDev[1];
   hiband[0] = hiband[1];
   loband[0] = loband[1];
   median[0] = median[1];
   

   for(int shift=limit; shift>=0; shift--)
   {
      //double CP = (Open[shift] + High[shift] + Low[shift] + 2 * Close[shift]) / 5;
      
      double sumx=0, sumy=0, sumxy=0, sumx2=0, sumy2=0;
      double m=0, yint=0, r=0;
      int n;

      // Linear Regression
      sumx = 0;
      sumy = 0;
      sumxy = 0;
      sumx2 = 0;
      sumy2 = 0;
      for(n = 0; n <= lp-1; n++)
      { 
         sumx = sumx + n;
         sumy = sumy + Close[shift + n];
         sumxy = sumxy + n * Close[shift + n];
         sumx2 = sumx2 + n * n;
         sumy2 = sumy2 + Close[shift + n] * Close[shift + n]; 
      }                      
      if(lp*sumx2-sumx*sumx==0)
         {m=(lp*sumxy-sumx*sumy)/(0.00000001);}
      else
         {m=(lp*sumxy-sumx*sumy)/(lp*sumx2-sumx*sumx);} 
      yint=(sumy+m*sumx)/lp;
      if(MathSqrt((lp*sumx2-sumx*sumx)*(lp*sumy2-sumy*sumy))==0)
         {r=(lp*sumxy-sumx*sumy)/(0.00000001);}
      else
         {r=(lp*sumxy-sumx*sumy)/MathSqrt((lp*sumx2-sumx*sumx)*(lp*sumy2-sumy*sumy));}
           
      LinReg[shift] = (yint-m*lp);
   }
   
   for (index = 0; index < limit; index++)
   {
      StdDev[index] = iStdDevOnArray(LinReg, 0, sp, 0, MODE_SMA, index);
   }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

